/********************************************
*  ArduinoでWi-Fiを使う
*　　Arduino Uno R4 WiFiを使用
*********************************************/
#include "WiFiS3.h"
#include <SparkFunBME280.h>
#include <Wire.h>
#define LED1 D2
#define LED2 D3
// アクセスポイント定数
char ssid[] = "Buffalo-?????";
char pass[] = "7tb7ksh?????";
// インスタンス生成
WiFiServer server(2000);
BME280 sensor;
// 変数定義
char currentLine[10];         // 受信バッファ
char i, c, Env[45];
int led =  LED_BUILTIN;
int status = WL_IDLE_STATUS;
float pres, temp, humi;

/***** 初期設定 **************/
void setup() {
  Wire.begin();
  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  Serial.begin(9600); 
  // センサ、OLED初期化
  sensor.settings.I2CAddress = 0x76;
  sensor.beginI2C();                        // センサ初期化
  // アクセスポイント接続
  while (status != WL_CONNECTED) {
    status = WiFi.begin(ssid, pass);
    delay(5000);                            // 5秒待ち
  }
  // 接続完了でIPアドレス表示
  Serial.println("");
  Serial.println("WiFi connected.");        // 接続完了メッセージ
  Serial.println("IP address: ");           // IPアドレスメッセージ
  Serial.println(WiFi.localIP());
  server.begin();                           // サーバリスン開始
}

/****** メインループ **********/
void loop() {
  WiFiClient client = server.available();   // listen for incoming clients
  while (client) {   
    i = 0;
    do{           // コマンド受信
      while (!client.available()); 
      c = client.read();                    // 2バイト受信
      Serial.write(c); 
      currentLine[i++] = c;                 // バッファに格納
    }while(c != 'E');                       // 終了文字まで繰り返し
    // 受信データ処理
    switch(currentLine[1]){
      case 'C': {    // LED制御要求の場合
        switch(currentLine[2]){             // LED区別
          case '1':     // LED1 オンオフ制御
            if(currentLine[3] == '1') digitalWrite(LED1, HIGH);
            else digitalWrite(LED1, LOW);
            break;
          case '2':     // LED2オンオフ制御
            if(currentLine[3] == '1') digitalWrite(LED2, HIGH);
            else digitalWrite(LED2, LOW);
            break;          
          default: break;
        }
        break;
      }
      case 'T': {   // 計測要求の場合
        pres = sensor.readFloatPressure()/100.0;  // 気圧データ取得
        temp = sensor.readTempC();                // 温度データ取得
        humi = sensor.readFloatHumidity();        // 湿度データ取得
        // 3個のデータをJSONフォーマットで送信
        sprintf(Env, "{\"Pres\":%3.2f, \"Temp\":%2.2f, \"Humi\":%2.2f}",pres/10, temp ,humi);
        client.write(Env, strlen(Env));
        Serial.write(Env);
        break;
      }
      default:  break; 
    } 
    // 終了
    client.stop();
  }
}


