//*****************************************
//  Wi-FiでパソコンとArduino Uno間通信
//　　BME280センサ
//****************************************
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
// BME280のインスタンス生成
Adafruit_BME280 bme;
// 変数定義
float temp;
float pres;
float humi;
char Buf[64], p[7], t[5], h[5];
/***********************************************
*  受信しながら文字列を探す関数
*   target：探す文字列　timeout　探す許容時間
***********************************************/
bool FindString(String target, uint32_t timeout)
{
    String data;
    char a, flag;
    unsigned long start = millis();
  
    flag = 0;                             // 発見フラグリセット
    while (millis() - start < timeout) {  // 指定タイムアウトまで繰り返す
      while(Serial1.available() > 0) {     // 受信ありの間繰り返す
        a = Serial1.read();                // 1文字受信
        if(a == '\0') continue;           // 0x00なら無視
          data += a;                      // 受信バッファに追加
      }
      if (data.indexOf(target) != -1) {   // 文字列を探す
          flag = 1;                       // 発見した
          break;                          // 強制終了
      }   
    }
    if(flag == 1)                         // 発見した場合
      return true;
    else
      return false;                       // 発見できなかった場合
}
//************ 初期設定　*********
void setup() {
  Serial1.begin(115200);                   // UART有効化
  bool status;
  status = bme.begin(0x76);               // センサ有効化
  while (!status) {
    Serial.println("BME280 sensorが使えません");
    delay(1000);
  }
  // APと接続確立
  Serial1.print("AT+CWMODE=1\r\n");            // クライアントモード
  do{                 // GOT IPまで繰り返す
    delay(1000);
    Serial1.print("AT+CWJAP=\"Buffalo-G-????\",\"7tb7ksh8?????\"\r\n");
  }while(FindString("GOT IP",6500)==false);
}

//******** メインループ *********
void loop() {
  // BME280センサからデータ読み出し
  pres=bme.readPressure() / 1000.0F;    //KPaに変換
  temp=bme.readTemperature();
  humi=bme.readHumidity(); 
  // 送信データ編集 
  dtostrf(pres, 6, 2, p);               // 気圧を文字列に変換　6桁で小数2桁
  dtostrf(temp, 4, 1, t);               // 温度を文字列に変換　4桁で小数1桁
  dtostrf(humi, 4, 1, h);               // 湿度を文字列に変換　4桁で小数1桁
  sprintf(Buf, "{\"Pres\":%s,\"Temp\":%s,\"Humi\":%s}",p, t, h);
  // PCとTCP接続実行　CONNECTまで繰り返す
  do{
    delay(1000);
    Serial1.print("AT+CIPSTART=\"TCP\",\"192.168.11.2\",9000\r\n");
  }while(FindString("CONNECT",500)==false);
  // メッセージ送信実行
  delay(1000);
  Serial1.print("AT+CIPSEND=39\r\n");    // 送信開始コマンド
  delay(2000);                          // 応答待ち
  Serial1.print(Buf);                    // データ送信
  delay(1000);                          // 応答待ち
  // PCとの接続解除
  Serial1.print("AT+CIPCLOSE\r\n");
  if(FindString("OK",500)==false){Serial1.println("NG\r\n");}
  delay(30000);      // 30秒間隔
}