/*****************************************
*  WiFiでLED制御と環境データ送信するサーバ
*　クライアントはNode-RED
******************************************/
#include <WiFi.h>   // for I2C
#include <Wire.h>
#include <SparkFunBME280.h>
#include <Adafruit_SSD1306.h>
// インスタンス定義
Adafruit_SSD1306 display(128, 32, &Wire, -1);
WiFiServer server(2000);
BME280 sensor;
hw_timer_t * timer = NULL;
//定数定義
const char* ssid     = "Buffalo-G-????";
const char* password = "7tb7ksh8????;
#define LED1 D0
#define LED2 D1
#define LED3 D2
#define LED4 D10
// 変数定義
char currentLine[10];         // 受信バッファ
char c;
int i, Flag;
char State[45], Env[45];
char state1,state2,state3,state4;
float pres, temp, humi;
//*** タイマ割り込み処理　********
void onTime(){
  Flag = true;
}
//***** セットアップ  *******/
void setup(){
  Wire.begin();                               // I2C有効化
  Serial.begin(115200);
  pinMode(LED1, OUTPUT);                      // LEDピン設定
  pinMode(LED2, OUTPUT);
  pinMode(LED3, OUTPUT);
  pinMode(LED4, OUTPUT);  
  // タイマの初期設定　2秒間隔
  timer = timerBegin(0, 8000, true);          // クロック80MHz　0.1ms
  timerAttachInterrupt(timer, &onTime, true); // 割り込み処理関数指定
  timerAlarmWrite(timer, 20000, true);        // 時間設定　2秒繰り返し
  timerAlarmEnable(timer);                    // タイマ動作開始
   // センサ、OLED初期化
  sensor.settings.I2CAddress = 0x76;
  sensor.beginI2C();                          // センサ初期化
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // SSID初期化
  //**** ネットワークとの接続開始　APと接続 ***
  Serial.println(); 
  Serial.println();
  Serial.print("Connecting to ");             // モニタメッセージ
  Serial.println(ssid);
  WiFi.begin(ssid, password);                 // 雪像相手メッセージ
  // 接続完了まで繰り返す
  while (WiFi.status() != WL_CONNECTED) {     // 接続確認
      delay(500);
      Serial.print(".");
  }
  // 接続完了でIPアドレス表示
  Serial.println("");
  Serial.println("WiFi connected.");        // 接続完了メッセージ
  Serial.println("IP address: ");           // IPアドレスメッセージ
  Serial.println(WiFi.localIP());
  // サーバ動作開始
  server.begin();
}
//****　メインループ　******
void loop(){
  // 割り込みごとにセンサデータOLED表示
  if(Flag == true){                       // 割り込みフラグオンの場合
    Flag = false;                         // 割り込みフラグクリア
    // OLED描画条件指定
    display.clearDisplay();               // 表示クリア
    display.setTextSize(1);               // 出力する文字の大きさ
    display.setTextColor(WHITE);          // 出力する文字の色 
    // センサデータOLED表示 
    display.setCursor(0, 0);              // 表示開始位置をホームにセット
    display.print("Press = ");            // 気圧見出し
    pres = sensor.readFloatPressure()/100.0;
    display.print(pres, 1);               // 気圧表示
    display.println(" hPa");              // 気圧の単位表示
    display.setCursor(0, 11);             // 2行目指定
    display.print("Temp  = ");            // 温度の見出し
    temp = sensor.readTempC();            // 温度データ取得
    display.print(temp, 2);               // 温度表示
    display.println("  DegC");            // 温度の単位表示
    display.setCursor(0, 22);             // 3行目
    display.print("Humi  = ");            // 湿度の見出し
    humi = sensor.readFloatHumidity();    // 湿度データ取得
    display.print(humi, 2);               // 湿度表示
    display.println("  %RH");             // 湿度の単位表示
    display.display();                    // 表示実行 
  }
  // ****リッスンモード開始 *****
  WiFiClient client = server.available();// 接続あり
  // 送受信繰り返し
  while(client) {                         // 受信ありの場合
    i = 0;                                // インデックスリセット
    do{ 
      while(!client.available());         // 受信ありの間繰り返す
      c = client.read();                  // 1バイト受信
//      Serial.write(c);                    // モニタへ出力
      currentLine[i++] = c;               // バッファへ格納
    }while(c != 'E');                     // E文字受信で完了
    //**** 文字列の処理　****
    switch(currentLine[1]){               // バッファ2文字目で分岐
      // LED制御コマンドの場合
      case 'C':
        switch(currentLine[2]){           // 3バイト目の文字で分岐
          case '1':                       // 1の場合LED1制御
            if(currentLine[3] == '1') digitalWrite(LED1, HIGH);
            else digitalWrite(LED1, LOW);
            break;
          case '2':                       // 2の場合LED2制御
            if(currentLine[3] == '1') digitalWrite(LED2, HIGH);
            else digitalWrite(LED2, LOW);
            break;      
          case '3':                       // 3の場合LED3制御
            if(currentLine[3] == '1') digitalWrite(LED3, HIGH);
            else digitalWrite(LED3, LOW);
            break;
          case '4':                       // 4の場合LED4制御
            if(currentLine[3] == '1') digitalWrite(LED4, HIGH);
            else digitalWrite(LED4, LOW);
            break;
          default: break;
        }
        // 制御結果の一括返信
        state1  = digitalRead(LED1) == 0 ? '0' : '1'; 
        state2 = digitalRead(LED2) == 0 ? '0' : '1'; 
        state3 = digitalRead(LED3) == 0 ? '0' : '1'; 
        state4 = digitalRead(LED4) == 0 ? '0' : '1'; 
        // JSONフォーマットで返信
        sprintf(State, "{\"D1\":%c, \"D2\":%c, \"D3\":%c, \"D4\":%c}",state1,state2,state3,state4);
        client.write(State, strlen(State));
        break;
      // センサ計測コマンドの場合
      case 'T':
        // 3個のデータをJSONフォーマットで送信
        sprintf(Env, "{\"Pres\":%3.2f, \"Temp\":%2.2f, \"Humi\":%2.2f}",pres/10, temp ,humi);
        client.write(Env, strlen(Env));
        break;
      default:
        break;
    }
    // 送受信終了　APとの接続は保持
    client.stop();
  }
}
