#************************************
#  Raspberry Pi Pico Wi-Fi Test
#  BME280 のデータを送信
#  PC側はNodeREDで30秒ごとにデータ要求
#************************************
from machine import Pin, I2C, Timer
from bme280 import BME280
import time
import network
import socket
#BME280 Sensor設定
i2c = I2C(1, sda=Pin(18), scl=Pin(19) )
bme = BME280(i2c=i2c)
#Wi-FiのSSIDとパスワード設定
ssid = 'Buffalo-?????'
password = '7tb7ksh?????'

#***** 初期設定 ************************
#Wi-Fi接続開始
wlan = network.WLAN(network.STA_IF)
wlan.active(True)
wlan.connect(ssid, password)
# WiFi接続完了待ち　3秒間隔
max_wait = 10
while max_wait > 0:
    if wlan.status() < 0 or wlan.status() >= 3:
        break
    max_wait -= 1
    print('waiting for connection...')
    time.sleep(3)
# 接続失敗の場合
if wlan.status() != 3:
    raise RuntimeError('network connection failed')
else:
    print('Connected')
    status = wlan.ifconfig()
    print( 'ip = ' + status[0] )   
# 接続成功、ソケットをオープンし、クライアント接続待ち
addr = socket.getaddrinfo('0.0.0.0', 2000)[0][-1]
s = socket.socket()
s.bind(addr)
s.listen(1)
print('listening on', addr)

#******* メインループ ************************
while True:
    try:
        # クライアント接続、メッセージ受信
        cl, addr = s.accept()
        print('client connected from', addr)
        #メッセージ受信しバイトから文字列に変換
        request = cl.recv(64)
        print(request)
        rcv = request.decode('utf8')
        
        # センサ計測コマンド(STE)の場合
        if(rcv[1] == 'T'):
            #BME280からデータ取得、補正変換
            tmp = bme.read_compensated_data()[0]/100
            pre = bme.read_compensated_data()[1]/256000
            hum = bme.read_compensated_data()[2]/1024
            Env = "{\"Pres\":"+str(pre)+",\"Temp\":"+str(tmp)+",\"Humi\":"+str(hum)+"}" 
            print(Env)
            #クライアントに返送
            cl.send(Env)
        cl.close()         
    except OSError as e:
        cl.close()
        print('connection closed')
        