//*********************************
//　ArduinoでI2C接続のセンサを使う
//　　I2C_Sensors
//　温湿度センサ　SHT31
//　気圧センサ　　LPS331AP
//　CO2センサ　　 CCS811
//　液晶表示器　　AQM1602
//********************************
#include <Wire.h>             // I2C用
#include "ST7032_asukiaaa.h"
#include "Adafruit_SHT31.h"
#include "LPS.h"
#include "Adafruit_CCS811.h"
// インスタンス生成
ST7032_asukiaaa lcd;          // 液晶表示器
Adafruit_SHT31 sht31 = Adafruit_SHT31();
LPS ps;
Adafruit_CCS811 ccs;
// 変数定義
float temp, humi, pres, co2;

//***** 初期設定 *******
void setup() {
  Serial.begin(9600);
  // LCD setting
  lcd.begin(16, 2);           // 16文字2行
  lcd.setContrast(30);        // コントラスト設定
  // SHT31センサ接続
  if(!sht31.begin(0x44)){
    Serial.println("Couldn't find sht31");
    while(1);
  }
  // 気圧センサ接続
  if(!ps.init()){
    Serial.println("Failed to autodetect pressure sensor.");
    while(1);
  }
  ps.enableDefault();
  // CO2センサ接続
  if(!ccs.begin()){
    Serial.println("Failed to start sensor");
    while(1);
  }
  while(!ccs.available());
}

//****** メインループ *****
void loop() {
  // 温湿度センサから読み出し
  temp = sht31.readTemperature();
  humi = sht31.readHumidity();
  // 温湿度表示編集と出力
  lcd.setCursor(0, 0);    // 1行目指定
  lcd.print(temp, 1);     // 温度表示
  lcd.print("\xDF");      // °C表示
  lcd.print("C");
  lcd.print("   ");
  lcd.print(humi, 1);     // 湿度表示
  lcd.print("%RH  ");

  // 気圧センサから読み出し
  pres = ps.readPressureMillibars();
  // 気圧編集と表示
  lcd.setCursor(0, 1);    // 2行目指定
  lcd.print(pres, 0);     // 気圧表示
  lcd.print("hPa  ");

  // CO2センサから読み出し
  if(ccs.available()){
    if(!ccs.readData()){  // 読み出し
      co2 = ccs.geteCO2();// CO2取り出し
      lcd.print(co2, 0);  // CO2表示
      lcd.print("ppm  ");
    }
  }
  else{
    Serial.println("Error");
  }
  delay(3000);            // 3秒間隔
}
