/***********************************
* M5Core2の例題　センサの接続例
*　Groveセンサを2種類接続し
*　液晶表示器に表示
*  CO２濃度でLED色表示
***********************************/
#include <M5Core2.h>
#include "SparkFun_SCD4x_Arduino_Library.h"
#include "M5_ENV.h"
#include "M5_EXTIO2.h"
// インスタンス生成
SCD4x mySensor;
SHT3X sht30;
QMP6988 qmp6988;
M5_EXTIO2 extio;
//変数定義
float pres, temp, humi;
uint16_t coto;
//** 初期設定 ***********
void setup() {
  Serial.begin(9600);
  M5.begin(true, false, true, true);  //LCD,SD,UART,I2C
  // センサ接続確認と初期化
  if (mySensor.begin() == false){
    Serial.println(F("Sensor not detected. Freezing..."));
    while (1);
  }
  extio.begin(&Wire, 21, 22, 0x45); // 拡張IO
  extio.setAllPinMode(DIGITAL_OUTPUT_MODE);
  qmp6988.init();
  // LCD初期化
  M5.Lcd.fillScreen(BLACK);
  M5.Lcd.setTextFont(4);  //26pxサイズ指定
}

//*** メインループ **********
void loop() {
  // ENV3制御
  pres = qmp6988.calcPressure()/100;  // hPa単位化
  if(sht30.get() == 0){
    temp = sht30.cTemp;
    humi = sht30.humidity;
  }
  M5.Lcd.setCursor(0, 0); 
  M5.lcd.setTextColor(GREEN, BLACK);  
  M5.lcd.printf("****  ENV 3  Sensor  ****");
  M5.lcd.setTextColor(YELLOW, BLACK); 
  M5.Lcd.setCursor(20, 35);
  M5.lcd.printf("Pres: %4.1f hPa", pres);
  M5.lcd.setCursor(20, 70);
  M5.lcd.printf("Temp: %2.1f DegC", temp);
  M5.lcd.setCursor(20, 105);
  M5.lcd.printf("Humi: %2.1f %%RH", humi);
  //SCD4x CO2 sensor制御
  if (mySensor.readMeasurement()){  // データ有効の場合
    coto = mySensor.getCO2();
    // LEDの表示
    if(coto > 2500){
      extio.setDigitalOutput(7, LOW);   // Green
      extio.setDigitalOutput(5, LOW);   // Blue
      extio.setDigitalOutput(3, HIGH);  // Red
    }
    else if(coto > 1500){
      extio.setDigitalOutput(7, HIGH);  // Green
      extio.setDigitalOutput(5, LOW);   // Blue
      extio.setDigitalOutput(3, HIGH);  // Red
    }
    else{
      extio.setDigitalOutput(7, LOW);   // Green
      extio.setDigitalOutput(5, HIGH);  // Blue
      extio.setDigitalOutput(3, LOW);   // Red
    }
    // 液晶表示器の表示
    M5.lcd.setTextColor(CYAN, BLACK); 
    M5.Lcd.setCursor(0, 160);
    M5.lcd.printf("****  SCD4x Sensor  ****");
    M5.lcd.setTextColor(YELLOW, BLACK);
    M5.Lcd.setCursor(20, 195);
    M5.Lcd.printf("CO2 : %4d ppm ", coto);
  }
  delay(2000); //2秒間隔
}
