/*************************************
* M5Stack Core2の例題
*　室内環境をWi-Fiで送信
*　GROBEの環境センサとCO2センサを使用
*　拡張I/OでLED制御　
*************************************/
#include <M5Core2.h>
#include "SparkFun_SCD4x_Arduino_Library.h"
#include "M5_ENV.h"
#include "M5_EXTIO2.h"
#include <WiFi.h>
// インスタンス生成
SCD4x mySensor;
SHT3X sht30;
QMP6988 qmp6988;
M5_EXTIO2 extio;
hw_timer_t *timer1 = NULL;    // 割り込み用タイマ
WiFiServer server(2000);
//変数定義
char currentLine[10];         // Wi-Fi受信バッファ
char c;
int i, Flag, coto;
char Env[64], state;          // Wi-Fisousin バッファ
float pres, temp1, temp, humi;
// アクセスポイント定数
const char* ssid     = "Buffalo-G-????";
const char* password = "7tb7ksh?????";

/*************************
* タイマ割り込み処理
*************************/
void Disp(void){
  Flag = 1;             // フラグセットのみ
}

//***** 初期設定 ***********
void setup() {
  M5.begin(true, false, true, true);          // LCD,SD,UART,I2C 

  // インターバル割り込みタイマ設定
  timer1 = timerBegin(0, 8000, true);         // 0.1ms period
  timerAttachInterrupt(timer1, &Disp, true);
  timerAlarmWrite(timer1, 20000, true);       // 2sec周期
  timerAlarmEnable(timer1);                   // 割り込み許可
  // センサ初期化
  if (mySensor.begin() == false){
    Serial.println(F("Sensor not detected. Freezing..."));
    while (1);
  }
  extio.begin(&Wire, 21, 22, 0x45);           // 拡張IO初期化
  qmp6988.init();
  // LCD初期化
  M5.Lcd.fillScreen(BLACK);
  M5.Lcd.setTextFont(4);      //26pxサイズ指定
  // Wi-Fi接続処理開始
  WiFi.mode(WIFI_STA);
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  // APと接続
  WiFi.begin(ssid, password);             // 接続実行
  while (WiFi.status() != WL_CONNECTED) { // 接続待ち
      delay(500);                         // 0.5秒間隔
      Serial.print(".");
  }
  // IPアドレス表示
  Serial.println("");
  Serial.println("WiFi connected.");  // 接続メッセージ
  Serial.println("IP address: ");     // 見出し
  Serial.println(WiFi.localIP());     // IPアドレス
  // サーバ開始
  server.begin();
}
//*** メインループ **********
void loop() {
  // 2秒ごとに液晶表示器に表示実行（タイマ割り込み）
  if(Flag == 1){
    Flag = 0;
    // ENV3制御
    pres = qmp6988.calcPressure()/100;    // hPa単位化
    if(sht30.get() == 0){
      temp = sht30.cTemp;
      humi = sht30.humidity;
    }
    // 液晶表示器の表示実行
    M5.Lcd.setCursor(0, 0); 
    M5.lcd.setTextColor(GREEN, BLACK);    // 文字緑
    M5.lcd.printf("****  ENV 3  Sensor  ****");
    M5.lcd.setTextColor(YELLOW, BLACK);   // 文字黄
    M5.Lcd.setCursor(20, 35);
    M5.lcd.printf("Pres: %4.1f hPa", pres);
    M5.lcd.setCursor(20, 70);
    M5.lcd.printf("Temp: %2.1f DegC", temp);
    M5.lcd.setCursor(20, 105);
    M5.lcd.printf("Humi: %2.1f %%RH", humi);
    //SCD4x CO2 sensor制御
    if (mySensor.readMeasurement()){  // データ有効の場合
      coto = mySensor.getCO2();
      // LEDの表示
      extio.setAllPinMode(DIGITAL_OUTPUT_MODE); 
      if(coto > 2000){
        extio.setDigitalOutput(7, LOW);   // Green
        extio.setDigitalOutput(5, LOW);   // Blue
        extio.setDigitalOutput(3, HIGH);  // Red
        state = 3;
      }
      else if(coto > 1000){
        extio.setDigitalOutput(7, HIGH);  // Green
        extio.setDigitalOutput(5, LOW);   // Blue
        extio.setDigitalOutput(3, HIGH);  // Red
        state = 2;
      }
      else{
        extio.setDigitalOutput(7, LOW);   // Green
        extio.setDigitalOutput(5, HIGH);  // Blue
        extio.setDigitalOutput(3, LOW);   // Red
        state = 1;
      }
      // 液晶表示器の表示
      M5.lcd.setTextColor(CYAN, BLACK); 
      M5.Lcd.setCursor(0, 160);
      M5.lcd.printf("****  SCD4x Sensor  ****");
      M5.lcd.setTextColor(YELLOW, BLACK);
      M5.Lcd.setCursor(20, 195);
      M5.Lcd.printf("CO2 : %4d ppm ", coto);
    }
  }
   //*** PCからの接続待ち ****
  WiFiClient client = server.available(); // PCのインスタンス生成
  // PCからのコマンド受信
  while(client) {                         // 接続中繰り返し
    i = 0;
    do{ 
      while(!client.available());         // 受信レディー待ち
      c = client.read();                  // 1バイト受信
//      Serial.write(c);                    // モニタへ出力
      currentLine[i++] = c;               // バッファへ格納
    }while(c != 'E');                     // 文字E受信で終了
    // コマンド処理　JSON形式でデータ返送
    if(currentLine[1] == 'T'){     
      sprintf(Env, "{\"Pres\":%4.0f, \"Temp\":%2.1f, \"Humi\":%2.1f,\"CO2\":%4d,\"LED\":%d}",pres, temp ,humi, coto, state);
      client.write(Env, strlen(Env));
      client.stop();
    }
  }
}
