#**************************************
#  ラズパイPicoの制御プログラム
#　　I2CでSHT31、LPF25、SSD1306を接続
#　　Wi-FiでPCに接続　Node-REDで表示
#**************************************
from machine import Pin, I2C, Timer
import ssd1306
import time
import network
import socket
#**** インスタンス生成 *************
#GPIO
led1 = Pin(2, Pin.OUT)
led2 = Pin(4, Pin.OUT)
led3 = Pin(6, Pin.OUT)
led4 = Pin(8, Pin.OUT)
#インター×タイマの生成
intervaltimer = Timer()
#I2Cのインスタンス生成
i2c = I2C(1, sda=Pin(18), scl=Pin(19))
#OLEDのインスタンス生成
display = ssd1306.SSD1306_I2C(128, 32, i2c)
#Wi-FiのSSIDとパスワード定義
ssid = 'Buffalo-G-????'
password = '7tb7ksh8?????'
#センサ用コマンドデータ
send = bytearray([0x2C, 0x06])
setting = bytearray([0x20, 0x90])
buf = bytearray([0xA8])
#気圧センサ初期設定
i2c.writeto(0x5C, setting)

#*** タイマ割り込み処理関数 ********
# 2秒ごとにセンサデータをOLEDに表示
def isrFunc(timer):
    global tmp, pre, hum
    #センサ計測トリガ後温湿度入力
    i2c.writeto(0x45, send)
    time.sleep(0.5)
    rcv =i2c.readfrom(0x45, 6)
    #データ変換
    tmp = rcv[0]<<8 | rcv[1]
    hum = rcv[3]<<8 | rcv[4]
    tmp = -45+175*(tmp/(2**16-1))
    hum = 100*hum/(2**16-1)
    #気圧センサ 0x28レジスタから連続3バイト読み出し
    i2c.writeto(0x5C, buf)    
    rcv = i2c.readfrom(0x5C, 3)
    pre = (rcv[2]<<16 | rcv[1]<<8 | rcv[0]) / 4096.0
    #OLEDに表示
    display.fill(0)
    display.text('Pres='+str('{:.1f}'.format(pre))+' hPa', 0, 0, 1)
    display.text('Temp='+str('{:.2f}'.format(tmp))+' DegC', 0, 11, 1)
    display.text('Humi='+str('{:.2f}'.format(hum))+' %RH ', 0, 22, 1)
    display.show() 

#******* 初期設定 ************************
#タイマ動作開始　2秒間隔
intervaltimer.init(period=2000, callback=isrFunc)

#**** アクセスポイントとの接続 **************
#アクセスポイント接続開始
wlan = network.WLAN(network.STA_IF)
wlan.active(True)
wlan.connect(ssid, password)
# アクセスポイント接続完了待ち　3秒間隔
max_wait = 10
while max_wait > 0:
    if wlan.status() < 0 or wlan.status() >= 3:
        break
    max_wait -= 1
    print('waiting for connection...')
    time.sleep(3)
# 接続失敗の場合
if wlan.status() != 3:
    raise RuntimeError('network connection failed')
#接続成功した場合 IPアドレス表示
else:
    print('Connected')
    status = wlan.ifconfig()
    print( 'ip = ' + status[0] )    
# 接続成功、サーバ動作開始、クライアント接続待ち
addr = socket.getaddrinfo('0.0.0.0', 2000)[0][-1]
s = socket.socket()
s.bind(addr)
s.listen(1)
#print('listening on', addr)


#**** メインループ *****************
# クライアント接続、メッセージ受信
while True:
    try:
        #クライアントインスタンス生成
        cl, addr = s.accept()
#        print('client connected from', addr)
        #メッセージ受信しバイトから文字列に変換
        request = cl.recv(64)
#        print(request)
        rcv = request.decode('utf8')
        #***** 文字列の処理、２文字目で分岐 ****
        #制御コマンドの場合 LED制御
        if(rcv[1] == 'C'):
            if(rcv[2] == '1'):
                if(rcv[3] == '1'):
                    led1.value(1)
                else:
                    led1.value(0)
            elif(rcv[2] == '2'):
                if(rcv[3] == '1'):
                    led2.value(1)
                else:
                    led2.value(0) 
            elif(rcv[2] == '3'):
                if(rcv[3] == '1'):
                    led3.value(1)
                else:
                    led3.value(0)
            elif(rcv[2] == '4'):
                if(rcv[3] == '1'):
                    led4.value(1)
                else:
                    led4.value(0)
            # 応答送信
            state1 = "{\"D1\":" + str(led1.value())
            state2 = ",\"D2\":" + str(led2.value())
            state3 = ",\"D3\":" + str(led3.value()) 
            state4 = ",\"D4\":" + str(led4.value())+"}"
            State = state1 + state2 + state3 + state4
  #          print(State)
            #クライアントに返送
            cl.send(State)
 
        # センサ計測コマンドの場合
        elif(rcv[1] == 'T'):
            Env = "{\"Pres\":"+str(pre)+",\"Temp\":"+str(tmp)+",\"Humi\":"+str(hum)+"}" 
  #          print(Env)
            #クライアントに返送
            cl.send(Env)
        # 接続終了 
        cl.close()    
    #例外の場合　クライアントとの接続終了   
    except OSError as e:
        cl.close()
        print('connection closed')
        

