#coding: utf-8
#************************************
# Raspberry Pi 4B sample program
#  I2C:BME280   SPI:MSP2807
#  Display Temp, Humi, Pres
#************************************/
from adafruit_rgb_display.ili9341 import ILI9341
from PIL import Image, ImageDraw, ImageFont
from busio import SPI
from digitalio import DigitalInOut
import board
import time
#***** LCDの初期設定 ******************
# LCD用ピンの設定
cs_pin = DigitalInOut(board.D8)
dc_pin = DigitalInOut(board.D25)
rst_pin = DigitalInOut(board.D24)
# SPIバスの設定
spi = SPI(clock=board.SCK, MOSI=board.MOSI, MISO=board.MISO)
# ILI9341 LCDのインスタンス生成
display = ILI9341(
    spi,
    cs=cs_pin, dc=dc_pin, rst=rst_pin,
    width=240, height=320,
    rotation=90,
    baudrate=24000000
)
# 色名称の定義
COLOR_WHITE   = "#FFFFFF"
COLOR_BLACK   = "#000000"
COLOR_RED     = "#FF0000"
COLOR_GREEN   = "#00FF00"
COLOR_BLUE    = "#0000FF"
COLOR_YELLOW  = "#FFFF00"
COLOR_CYAN    = "#00FFFF"
COLOR_MAGENTA ="#FF00FF"
# グラフィックのインスタンス生成
image = Image.new("RGB", (320, 240))
# 描画オブジェクトの生成
draw = ImageDraw.Draw(image)
# フォントの入手　ipaexfont-gothic font
font1 = ImageFont.truetype("/usr/share/fonts/opentype/ipaexfont-gothic/ipaexg.ttf", 48)
font2 = ImageFont.truetype("/usr/share/fonts/opentype/ipaexfont-gothic/ipaexg.ttf", 32)
font3 = ImageFont.truetype("/usr/share/fonts/opentype/ipaexfont-gothic/ipaexg.ttf", 20)

#****** BME280センサの初期設定 ********
import smbus2
import bme280
# BME280用のピン指定
port = 1
address = 0x76
bus = smbus2.SMBus(port)
# 較正データの取得
calibration_params = bme280.load_calibration_params(bus, address)

#******* メインループ **********************
while True:   
    # センサからデータ取得
    data = bme280.sample(bus, address, calibration_params)
    # 全画面消去　黒
    draw.rectangle((0, 0, 320, 240), outline=0, fill=(0, 0, 0))
    # 見出しの表示
    draw.text((8, 8), "＊＊  室内環境  ＊＊", font=font2, fill=COLOR_YELLOW)     
    # 温度の見出し、データ、単位の表示
    draw.text((8, 64), "温度:", font=font2, fill=COLOR_MAGENTA)
    temp =format(float(data.temperature), '.1f')
    draw.text((96, 56), temp, font=font1, fill=COLOR_WHITE)
    draw.text((230, 64), "℃", font=font2, fill=COLOR_BLUE)
    # 湿度の見出し、データ、単位の表示
    draw.text((8, 120), "湿度:", font=font2, fill=COLOR_CYAN)
    temp =format(float(data.humidity), '.1f')
    draw.text((96, 112), temp, font=font1, fill=COLOR_WHITE)
    draw.text((230, 120), "%RH", font=font2, fill=COLOR_BLUE)    
    # 気圧の見出し、データ、単位の表示
    draw.text((8, 176), "気圧:", font=font2, fill=COLOR_GREEN)
    pres =format(float(data.pressure), '.0f')
    draw.text((96, 168), pres, font=font1, fill=COLOR_WHITE)
    draw.text((230, 176), "hPa", font=font2, fill=COLOR_BLUE)     
    # 実際の表示実行
    display.image(image)
    # 5秒待ち
    time.sleep(5)

