#************************************
#  Raspberry Pi Pico Ambient
#  BME280 のデータをAmbientに送信
#　　30秒ごとに送信
#************************************
from machine import Pin, I2C, Timer
from bme280 import BME280
import time
import network
import urequests
#BME280 Sensor設定
i2c = I2C(1, sda=Pin(18), scl=Pin(19) )
bme = BME280(i2c=i2c)
#Wi-FiのSSIDとパスワード設定
ssid = 'Buffalo-G-????'
password = '7tb7ksh8?????'
#POSTメッセージ用データ定義
url = 'https://ambidata.io/api/v2/channels/'+'?????'+'/data'
http_headers = {'Content-Type':'application/json'}
http_body = {'writeKey':'b87afa23729????', 'd1':0.0, 'd2':0.0, 'd3':0.0}

#***** 初期設定 ************************
# Wi-Fi接続開始
wlan = network.WLAN(network.STA_IF)
wlan.active(True)
wlan.connect(ssid, password)
# WiFi接続完了待ち　3秒間隔で繰り返す
max_wait = 10
while max_wait > 0:
    if wlan.status() < 0 or wlan.status() >= 3:
        break
    max_wait -= 1
    print('waiting for connection...')
    time.sleep(3)
# 接続失敗の場合
if wlan.status() != 3:
    raise RuntimeError('network connection failed')
# 正常接続の場合　IPアドレスを表示
else:
    print('Connected')
    status = wlan.ifconfig()
    print( 'ip = ' + status[0] )  
    
#******* メインループ ************************
while True:    
    #BME280からデータ取得、補正変換
    tmp = bme.read_compensated_data()[0]/100
    pre = bme.read_compensated_data()[1]/25600
    hum = bme.read_compensated_data()[2]/1024
    #Body部にデータセット
    http_body['d1'] = str(tmp)
    http_body['d2'] = str(hum)
    http_body['d3'] = str(pre)
    # Ambientに送信
    try:
        res = urequests.post(url, json=http_body, headers=http_headers)
        # Message 200 is OK
        print('HTTP State=', res.status_code)
    except Exception as e:
        print(e)
    res.close()
    # Wait 30sec
    time.sleep(30)
    
    
    
    