#************************************
#  Raspberry Pi Pico IFTTT
#  BME280 のデータをIFTTT経由
#  スプレッドシートへ追加する
#************************************
from machine import Pin, I2C, Timer
from bme280 import BME280
import time
import network
import urequests
#BME280 Sensor設定
i2c = I2C(1, sda=Pin(18), scl=Pin(19) )
bme = BME280(i2c=i2c)
#Wi-FiのSSIDとパスワード設定
ssid = 'Buffalo-G-????'
password = '7tb7ksh8?????'
#POSTメッセージ定義
url = 'http://maker.ifttt.com/trigger/Add_Data/with/key/dmmT_6fbmVbuF5b????'
header = {'Content-Type':'application/json'}
body = {'value1':0.0, 'value2':0.0, 'value3':0.0}

#***** 初期設定 ************************
#Wi-Fi接続開始
wlan = network.WLAN(network.STA_IF)
wlan.active(True)
wlan.connect(ssid, password)
# WiFi接続完了待ち　3秒間隔　10回繰り返す
max_wait = 10
while max_wait > 0:
    if wlan.status() < 0 or wlan.status() >= 3:
        break
    max_wait -= 1
    print('waiting for connection...')
    time.sleep(3)
# 接続失敗の場合　エラーメッセージ出力
if wlan.status() != 3:
    raise RuntimeError('network connection failed')
else:
    # 正常　IPアドレス出力
    print('Connected')
    status = wlan.ifconfig()
    print( 'ip = ' + status[0] )   

#******* メインループ ************************
while True:
    #BME280からデータ取得、補正変換
    temp = bme.read_compensated_data()[0]/100
    pres = bme.read_compensated_data()[1]/25600
    humi = bme.read_compensated_data()[2]/1024
    # Bodyにデータ追加
    body['value1'] = temp
    body['value2'] = humi
    body['value3'] = pres
    # IFTTTに送信
    try:
        res = urequests.post(url, json=body, headers=header)
        # Message 200 is OK
        print('HTTP State=', res.status_code)
    except Exception as e:
        print(e)
    res.close()
    # Wait 1min
    time.sleep(60)