/*****************************************
*  PubNub経由PCからコマンド受信
*　1分ごとの計測要求で温度、湿度、気圧を返送
*　LED制御コマンドでLEDオンオフ制御
******************************************/
#include <WiFi.h>
#include <SparkFunBME280.h>
#include <PubNub.h>
// インスタンス定義
BME280 sensor;
WiFiClient client;
//定数定義
const char *ssid = "Buffalo-G?????";
const char *password = "7tb7ksh?????";
const char pubkey[]  = "pub-c-da98eefa-4176-????????????????";
const char subkey[]  = "sub-c-221af365-7acd-????????????????";
char  t[7], h[7], p[7], message[60];
String msg;
/***** セットアップ  *******/
void setup(){
  pinMode(2, OUTPUT);                         // LED pin Output
  pinMode(3, OUTPUT);
  pinMode(10, OUTPUT);
  Wire.begin();                               // I2C有効化
  Serial.begin(115200);
  // センサ初期化
  sensor.settings.I2CAddress = 0x76;
  sensor.beginI2C();                          // センサ初期化
  /**** ネットワークとの接続開始　APと接続 ***/
  WiFi.begin(ssid, password);                 // 接続相手メッセージ
  // 接続完了まで繰り返す
  while (WiFi.status() != WL_CONNECTED) {     // 接続確認
      delay(500);
      Serial.print(".");
  }
  // 接続完了でIPアドレス表示
  Serial.println("");
  Serial.println("WiFi connected.");        // 接続完了メッセージ
  Serial.println("IP address: ");           // IPアドレスメッセージ
  Serial.println(WiFi.localIP());
  // PubNubと接続
  if(PubNub.begin(pubkey, subkey)){  
    Serial.println("Connect to PubNub");
  }
  else{
    Serial.println("Connection Fail");      // 接続失敗
  }
}
/*****  メインループ***********/
void loop() {
  // 常時サブスクライブ状態
  PubSubClient *sclient = PubNub.subscribe("Test");   // サブスクライブ実行
  if(sclient){                                        // 正常接続
    SubscribeCracker ritz(sclient);                   // 受信処理
    while(!ritz.finished()){                          // 受信データ最後まで
      ritz.get(msg);                                  // 受信データ取り出し
      if(msg.length() > 0){                           // 受信データありの場合
        Serial.println(msg);                          // モニタ出力
        // 計測要求チェック
        if(msg.indexOf("MES", 0) != -1){              // 計測要求の場合
          digitalWrite(10, HIGH);                     // 目印LED
          float temp = sensor.readTempC();            // 温度データ取得
          float humi = sensor.readFloatHumidity();    // 湿度データ取得
          float pres = sensor.readFloatPressure()/100.0;  // 気圧データ取得
          // 送信データ作成
          dtostrf(temp, 3, 1, t);                     // 温度
          dtostrf(humi, 3, 1, h);                     // 湿度
          dtostrf(pres, 5, 1, p);                     // 気圧
          // 送信データ生成
          sprintf(message, "{\"value1\":\"%s\",\"value2\":\"%s\",\"value3\":\"%s\"}\r\n", t, h, p);
          // パブリッシュ実行
          PubNonSubClient *pclient = PubNub.publish("Test", message);
          delay(100);  
          digitalWrite(10, LOW);                      // 目印オフ
          pclient->stop();                            // 送信クライアントクローズ   
        }
        //LED制御チェック
        else if(msg.indexOf("LED1", 0) != -1 ){       // LED1の場合
          if(msg.indexOf('1', 5) == -1){              // OFFの場合
            digitalWrite(2, LOW);
          }        
          else{                                       // ONの場合
            digitalWrite(2, HIGH);
          }
        }
        else if(msg.indexOf("LED2", 0) != -1){       // LED2の場合
          if(msg.indexOf('1', 5) == -1){             // OFFの場合
            digitalWrite(3, LOW);
          }        
          else{                                      // ONの場合
            digitalWrite(3, HIGH);
          }        
        }
      } 
    }
  }
}
